{***************************************************************************}
{                              UNIDAD DE FRASE

                                  Copyright (c) 1998 Marco Antonio Alvarado

Tiene rutinas para manejar las frases y palabras.

Una palabra es un smbolo o un grupo de smbolos diferentes del espacio.
Por ejemplo la frase:

  "La ecuacin es y = a*x+64"
   |  |        |  | | |||||
   |  |        |  | | ||||+--- 10 "sesenta y cuatro"
   |  |        |  | | |||+---- 9  "ms"
   |  |        |  | | ||+----- 8  "equis"
   |  |        |  | | |+------ 7  "por"
   |  |        |  | | +------- 6  "a"
   |  |        |  | +--------- 5  "es igual a"
   |  |        |  +----------- 4  "ye"
   |  |        +-------------- 3  "es"
   |  +----------------------- 2  "ecuacin"
   +-------------------------- 1  "la"

Esta frase contiene 10 palabras. Se puede apreciar que ciertos smbolos
individuales son palabras; mientras que otros smbolos se agrupan y estn
separados por espacios o por smbolos individuales.                         }
{***************************************************************************}

UNIT UFrase;

INTERFACE

  TYPE
    PString = ^STRING;

  FUNCTION CuentaPalabras (Frase : STRING) : Byte;
  FUNCTION CopiaPalabra (Frase : STRING; Lugar : Byte) : STRING;
  FUNCTION BajaLetra (Letra : Char) : Char;
  FUNCTION SubeLetra (Letra : Char) : Char;
  FUNCTION BajaFrase (Frase : STRING) : STRING;
  FUNCTION SubeFrase (Frase : STRING) : STRING;
  FUNCTION CreaFrase (Contenido : STRING) : PString;
  PROCEDURE EliminaFrase (VAR Frase : PString);

IMPLEMENTATION

  CONST
    Espacio = ' ';

  FUNCTION EsSimboloGrupo (Simbolo : Char) : Boolean;
  BEGIN
    EsSimboloGrupo :=
{Nmeros.}
      ((Simbolo >= '0') AND (Simbolo <= '9')) OR
{Letras.}
      ((Simbolo >= 'A') AND (Simbolo <= 'Z')) OR
      ((Simbolo >= 'a') AND (Simbolo <= 'z')) OR
{Letras especiales.}
      ((Simbolo >= '') AND (Simbolo <= '')) OR
      ((Simbolo >= '') AND (Simbolo <= ''));
  END;

  FUNCTION CuentaPalabras (Frase : STRING) : Byte;

    VAR
      Tamano : Byte ABSOLUTE Frase;
      Indice,
      Cantidad : Byte;

  BEGIN

    IF Frase = '' THEN CuentaPalabras := 0 ELSE
    BEGIN
      Indice := 1;
      Cantidad := 0;

      REPEAT

        IF Frase [Indice] = Espacio THEN
        BEGIN
{Salta los espacios.}
          REPEAT Inc (Indice) UNTIL (Indice > Tamano) OR
            (Frase [Indice] <> Espacio);
        END ELSE
        BEGIN
          Inc (Cantidad);

          IF EsSimboloGrupo (Frase [Indice]) THEN
          BEGIN
{Cuenta el grupo de smbolos.}
            REPEAT Inc (Indice) UNTIL (Indice > Tamano) OR
              (EsSimboloGrupo (Frase [Indice]) = False);
          END ELSE
          BEGIN
{Cuenta los dems smbolos de uno en uno.}
            Inc (Indice);
          END;

        END;

      UNTIL Indice > Tamano;

      CuentaPalabras := Cantidad;
    END;

  END;

  FUNCTION CopiaPalabra (Frase : STRING; Lugar : Byte) : STRING;

    LABEL
      Fin;

    VAR
      TamanoFrase : Byte ABSOLUTE Frase;
      Indice,
      Cantidad,
      Inicio,
      TamanoPalabra : Byte;

  BEGIN

    IF Frase = '' THEN CopiaPalabra := '' ELSE
    BEGIN
      Indice := 1;
      Cantidad := 0;

      REPEAT

        IF Frase [Indice] = Espacio THEN
        BEGIN
{Salta los espacios.}
          REPEAT Inc (Indice) UNTIL (Indice > TamanoFrase) OR
            (Frase [Indice] <> Espacio);
        END ELSE
        BEGIN
          Inc (Cantidad);

          IF EsSimboloGrupo (Frase [Indice]) THEN
          BEGIN

            IF Lugar = Cantidad THEN
            BEGIN
{Copia la palabra.}
              TamanoPalabra := 0;
              Inicio := Indice;

{Mide la palabra.}
              REPEAT
                Inc (Indice);
                Inc (TamanoPalabra);
              UNTIL (Indice > TamanoFrase) OR
                (EsSimboloGrupo (Frase [Indice]) = False);

              CopiaPalabra := Copy (Frase, Inicio, TamanoPalabra);
              GOTO Fin;
            END;

{Cuenta el grupo de smbolos.}
            REPEAT Inc (Indice) UNTIL (Indice > TamanoFrase) OR
              (EsSimboloGrupo (Frase [Indice]) = False);
          END ELSE
          BEGIN

            IF Lugar = Cantidad THEN
            BEGIN
{Copia la palabra.}
              CopiaPalabra := Frase [Indice];
              GOTO Fin;
            END;

{Cuenta los dems smbolos de uno en uno.}
            Inc (Indice);
          END;

        END;

      UNTIL Indice > TamanoFrase;

      CopiaPalabra := '';

      Fin:

    END;

  END;

  FUNCTION BajaLetra (Letra : Char) : Char;
  BEGIN

    CASE Letra OF
      'A'..'Z' : BajaLetra := Char (Byte (Letra) + 32);
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
      '' : BajaLetra := '';
    ELSE
      BajaLetra := Letra;
    END;

  END;

  FUNCTION SubeLetra (Letra : Char) : Char;
  BEGIN

    CASE Letra OF
      'a'..'z' : SubeLetra := Char (Byte (Letra) - 32);
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
      '' : SubeLetra := '';
    ELSE
      SubeLetra := Letra;
    END;

  END;

  FUNCTION BajaFrase (Frase : STRING) : STRING;

    VAR
      Tamano : Byte ABSOLUTE Frase;
      Indice : Byte;

  BEGIN
    FOR Indice := 1 TO Tamano DO
      Frase [Indice] := BajaLetra (Frase [Indice]);
    BajaFrase := Frase;
  END;

  FUNCTION SubeFrase (Frase : STRING) : STRING;

    VAR
      Tamano : Byte ABSOLUTE Frase;
      Indice : Byte;

  BEGIN
    FOR Indice := 1 TO Tamano DO
      Frase [Indice] := SubeLetra (Frase [Indice]);
    SubeFrase := Frase;
  END;

  FUNCTION CreaFrase (Contenido : STRING) : PString;

    VAR
      Tamano : Byte ABSOLUTE Contenido;
      Nueva : PString;

  BEGIN
    GetMem (Nueva, Byte (Contenido [0]) + 1);
    Move (Contenido, Nueva^, Tamano + 1);
    CreaFrase := Nueva;
  END;

  PROCEDURE EliminaFrase (VAR Frase : PString);
  BEGIN

    IF Frase <> NIL THEN
    BEGIN
      FreeMem (Frase, Byte (Frase^ [0]) + 1);
      Frase := NIL;
    END;

  END;

END.